#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
Importing modules
"""

# You can import python standard modules
import math
import numpy

# You can find out which functions and attributes
# are defined in a module.
# The built-in function dir() is used to find out which names a module defines.
# It returns a sorted list of strings:
print(dir(math))
help(math)
help(math.ceil)
help(math.floor)
help(math.sqrt)

print(math.ceil(3.7))  # => 4.0
print(math.floor(3.7))  # => 3.0
print(math.sqrt(16))  # => 4.0

# You can get specific functions from a module
from math import ceil, floor, sqrt

print(ceil(3.7))  # => 4.0
print(floor(3.7))  # => 3.0
print(sqrt(16))  # => 4.0

# You can import all functions from a module.
# Warning: this is not recommended
# Do NOT generally pollute your namespace with this:
from math import *

# Look at the namespace in the debugger

"""
Namespaces, scoping, and namespace collisions.

A Python statement can access variables in a local namespace and in the global
namespace.  If a local and a global variable have the same name, the local
variable shadows the global variable.

Variables are names (identifiers) that map to objects. A namespace is a
dictionary of variable names (keys) and their corresponding objects (values).
* A Python statement can access variables in a local namespace and in the
  global namespace. If a local and a global variable have the same name, the
  local variable shadows the global variable.
* Each function has its own local namespace. Class methods follow the same
  scoping rule as ordinary functions.

Each module has its own private symbol table, which is used as the global
symbol table by all functions defined in the module. Thus, the author of a
module can use global variables in the module without worrying about accidental
clashes with a user’s global variables. On the other hand, if you know what you
are doing you can touch a module’s global variables with the same notation used
to refer to its functions, modname.itemname.
"""

print("\n", locals())
print("\n", globals())

"""
The globals() and locals() functions can be used to return the names in the
global and local namespaces depending on the location from where they are
called.
* If locals() is called from within a function, it will return all the names
  that can be accessed locally from that function.
* If globals() is called from within a function, it will return all the names
  that can be accessed globally from that function.
"""

# What is in the base namespace?
# dir() does not list the names of built-in functions and variables. If you
# want a list of those, they are defined in the standard module builtins:
import builtins

print(dir(builtins))

# order matters (can overwrite another) don't use this:
print(dir(math))
from math import *

print(sin(3))
print(sqrt(3))

print(dir(numpy))
from numpy import *

# Show these change in the debugger variable view
print(sin(3))
print(sqrt(3))

# Instead the least likely to collide:
import math

# Still possible to accidentally collide
from math import sqrt

# You can change what imports are named:
import math as m
from math import sqrt as squareroot

print(math.sqrt(16) == m.sqrt(16) == squareroot(16))  # => True

print("\n", locals())
print("\n", globals())


def ok():
    """
    Imports are ok in functions.
    """
    from math import sqrt as square_root

    print(square_root(16))
    print(locals())
    print(globals())


# Imports exist in the scope of the function only.
ok()
# This was only in the scope of the function above
# square_root(16)
# NameError: name 'square_root' is not defined


def bad():
    """
    While imports are ok in functions, 
    they are not allowed in * form.
    """
    # This is not allowed
    # from math import *
    print(sqrt(16))


# +++++++++++++++ Cahoot 17.1
# https://mst.instructure.com/courses/45616/quizzes/48721
