#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
Python modules can be just ordinary Python files.
You can write your own, and import them.
The name of the module is the same as the name of the file.
Note: you skip the .py extension

Modules must be located in the search path, therefore you can:
* write your own modules within directories already defined in the search path
  (e.g. $HOME/.local/lib/python3.8/dist-packages).
  You may use symbolic links (on Linux) to keep the code somewhere else.
* modify the environment variable PYTHONPATH to include the directories
  containing the user-defined modules.
* On Linux/Unix, add the following line to a file read by the shell at startup
  (e.g. /etc/profile, .profile)
  export PYTHONPATH=$PYTHONPATH:/home/emma/user_defined_modules

You can put the module file in any directory of your choice and then modify
sys.path at run-time so that it contains that directory.

When the import mymodule statement is executed, the module mymodule is searched
in a given list of directories. This list includes a list of
installation-dependent default paths (e.g., /usr/lib/python) as well as the list
of directories specified by the environment variable PYTHONPATH.

When a module named spam is imported, the interpreter first searches for a
built-in module with that name.  If not found, it then searches for a file
named spam.py in a list of directories given by the variable sys.path.
sys.path is initialized from these locations:
* The directory containing the input script
  (or the current directory when no file is specified).
* PYTHONPATH (a list of directory names,
  with the same syntax as the shell variable PATH).
* The installation-dependent default directories.

After initialization, Python programs can modify sys.path.  The directory
containing the script being run is placed at the beginning of the search path,
ahead of the standard library path.  This means that scripts in that directory
will be loaded instead of modules of the same name in the library directory.

Interpreter differences (watch out!)
ipython3 tolerates directories relative to your cwd, for example:
sys.path.append('../../folder')

but python3 does not. It requires absolute references:
sys.path.append('/home/user/mymodules/folder')

Python standard is checked before your current directory.
"""
# This is the real math, despite the fact that we have math.py in this dir
import math

math.sqrt(16)

# This is your file (fiblib.py) in this current working directory of this script
import fiblib

# Trace this
print(fiblib.fib(4))

# details about module
print(dir(fiblib))
# Which did we write, and which are python's?
print(fiblib.__file__)
print(fiblib.__name__)
print(fiblib.__doc__)
help(fiblib)
# How does if __name__ == "__main__" work?

# import directly into global namespace
from fiblib import fib

print(fib(5))
# This works to import functions, classes, objects, whatever

from fiblib import fib as fibbing

print(fibbing(5))

# don't do this!
from fiblib import *

# to import a library named in a variable name:
student_module = "fiblib"
from importlib import import_module

mod = import_module(student_module)
print(mod.fib(5))

# To import from a particular folder:
import sys

print(sys.modules)
print(sys.path)

sys.path.append("subfolder")
import fiblib2

print(fiblib2.fib(4))

# if you want to avoid duplication on the path:
new_path = "subfolder"
if new_path not in sys.path:
    sys.path.append(new_path)

# To reload a module you've changed the source of:
from importlib import reload

reload(fiblib)

# In production code, you may want to warn users:
try:
    # Non-existent module
    import fiblib
except ImportError:
    print("Module not found")

# Another example
import mod

"""
Several objects are defined in mod.py:
    s (a string)
    a (a list)
    foo() (a function)
    Foo (a class)
"""
print(mod.s)
print(mod.a)
print(mod.foo(mod.s))
myobject = mod.Foo()
print(myobject)

# Notice what happens here
import mod
import mod

reload(mod)


# +++++++++++++++ Cahoot 17.2
# https://mst.instructure.com/courses/45616/quizzes/48722
